import argparse

from .main import main

if __name__ == "__main__":
    parser = argparse.ArgumentParser(
        description="Converts Obsidian notes to markdown export"
    )
    parser.add_argument(
        "--config", type=str, help="The config file", default="config.yaml"
    )
    args = parser.parse_args()

    main(args.config)



"""The role of this module is to create the correct pipeline for the conversion of the obsidian notes into markdown notes."""

from pydantic import BaseModel


class SourcesList(BaseModel):
    paths: list[str]
    images: list[str]


class Destination(BaseModel):
    base: str  # URL base o fthe files
    path: str  # where to store the files
    images: str  # where to store the images
    images_path: str  # where to store the images in the filesystem
    filesystem: str  # the location of the processed files


class Operation(BaseModel):
    name: str
    options: dict


class Config(BaseModel):
    sources: SourcesList
    output: Destination
    pipeline: list[Operation]



import os
import random

from obsidown import utils
from obsidown.config import Config
from obsidown.operations.base import MdFile, MdOperations
import copy


class UpdateFrontMatter(MdOperations):
    def __init__(self, config: Config, frontmatter: dict[str, str]):
        self.config = config

        self.new_metadata = frontmatter
        # TODO: this should be configured from args
        # self.new_metadata = {
        #     "layout": "page",
        #     "title": None,
        #     "permalink": None,
        #     "tags": "italian",
        # }

    def __call__(self, file: MdFile) -> MdFile:
        """Updates the frontmatter of the markdown files."""

        metadata = copy.deepcopy(self.new_metadata)

        if "title" in file.metadata:
            metadata["title"] = file.metadata["title"]
        else:
            no_extension = utils.remove_extension(os.path.basename(file.filename))
            metadata["title"] = no_extension

        if "url" in file.metadata:
            metadata["url"] = file.metadata["url"]
        # metadata["permalink"] = utils.to_kebab_case(
        #     self.config.output.path + "/" + no_extension
        # )

        # metadata["url"] = utils.to_kebab_case(
        #     self.config.output.path + "/" + no_extension
        # )

        if "language" in file.metadata:
            metadata["language"] = file.metadata["language"]

        if "tags" in file.metadata:
            metadata["tags"] = file.metadata["tags"]
        else:
            metadata["tags"] = ["no-tags"]

        if "summary" in file.metadata:
            metadata["summary"] = file.metadata["summary"]
            metadata["description"] = file.metadata["summary"]

        if "weight" in file.metadata:
            metadata["weight"] = file.metadata["weight"]
        else:
            # else we use the last commit time
            if "last_commit_time" in file.metadata:
                metadata["weight"] = utils.interpolate_weight(file.metadata["last_commit_time"])
            else:
                metadata["weight"] = random.randint(1, 50)  # play lottery lol

        return MdFile(
            metadata=metadata,
            contents=file.contents,
            references=file.references,
            filename=file.filename,
        )



"""Convert a citation format to a link citation format."""

from obsidown.operations.base import MdFile, MdOperations
import bibtexparser
import bibtexparser.middlewares as m
import bibtexparser.model as model
import re

bibdict: dict[str, model.Entry] | None = None


class RemoveTitleCurly(m.BlockMiddleware):
    """I don't know why the library adds the curcly braces authomatically,
    this middleware just removees them
    """

    def transform_entry(self, entry, *args, **kwargs):
        entry["title"] = entry["title"].replace("{", "").replace("}", "")
        return entry


class CitationConvert(MdOperations):
    def __init__(self, bibfile: str):
        """Load the bib file and store it in the object.
        bibfile can be a string or a path to the bibfile!?
        """
        # use cache
        global bibdict
        if bibdict is not None:
            self.bib = bibdict
            return

        with open(bibfile, "r") as f:
            string = f.read()

        layers = [
            RemoveTitleCurly(),
            m.SeparateCoAuthors(),  # Co-authors should be separated as list of strings
            m.SplitNameParts(),  # Individual Names should be split into first, von, last, jr parts
        ]

        db = bibtexparser.parse_string(string, append_middleware=layers)

        entries_dict = {}
        for entry in db.entries:
            entries_dict[entry.key] = entry

        self.bib = entries_dict
        bibdict = entries_dict

    def __call__(self, file: MdFile) -> MdFile:
        """Converts the citations in the markdown file to a link citation format."""
        new_contents = file.contents
        citation_key_set: set[str] = set()

        def convert_to_citation(match):
            citation_key = match.group(1)
            citation_key_set.add(citation_key)

            key = citation_key
            if key not in self.bib:
                raise ValueError(f"Key {key} not found in the bib file")

            entry = self.bib[key]

            citation_string = f"({self._format_citation(entry)})"
            if "url" in entry:
                return f'[{citation_string}]({entry["url"]})'
            else:
                print(f"WARNING: No url in the bib entry {entry.key}")

            return citation_string

        new_contents = re.sub(
            r"\[\[@([^\]]+?)(\|([^\]]+))?\]\]",
            convert_to_citation,
            new_contents,
        )

        # Should add config to see if needs to add a section of references at the end
        if len(citation_key_set) != 0:
            new_contents += "\n\n# References\n\n"
        for i, key in enumerate(citation_key_set):
            entry = self.bib[key]
            new_contents += f"[{i+1}] {self._format_long_citation(entry)}\n\n"

        return MdFile(
            metadata=file.metadata,
            contents=new_contents,
            references=file.references,
            filename=file.filename,
        )

    def _format_citation(self, entry: model.Entry):
        """Prints a citation using APA style"""
        final_string = self._format_citation_name(entry)
        final_string += f" {self.__format_date(entry)}"
        final_string = (
            final_string.strip()
        )  # in case format_date returns an empty string

        return final_string

    def __format_date(self, entry: model.Entry):
        """Formats the date of the citation"""
        if "date" not in entry:
            print(f"WARNING: No date in the bib entry {entry.key}")
            return ""
        # assume is in the format of %Y-%m-%d, or %Y-%m or %Y
        date_str = entry["date"]
        date_list = date_str.split("-")
        year = date_list[0]
        return year

    def _format_citation_name(self, entry: model.Entry):
        """Formats the citation name"""
        if "author" not in entry:
            raise ValueError("No author in the bib entry")

        def join_name_parts(name_parts: list[str]) -> str:
            return " ".join(name_parts)

        authors: list[m.NameParts] = entry["author"]

        final_string = f"{join_name_parts(authors[0].last)}"
        if len(authors) == 1:
            pass
        elif len(authors) == 2:
            final_string += f" & {join_name_parts(authors[1].last)}"
        else:
            final_string += " et al."

        return final_string

    def _format_long_citation(self, entry: model.Entry):
        final_string = self._format_citation_name(entry)
        if "url" in entry:
            final_string += f" [“{entry['title']}”]({entry['url']})"
        else:
            final_string += f" “{entry['title']}”"

        final_string += f" {self.__format_journal_type(entry)}"
        final_string += f" {self.__format_date(entry)}"
        return final_string

    def __format_journal_type(self, entry: model.Entry):
        """Formats the journal citation type"""

        match entry.entry_type:
            case "article":
                return self.__format_article(entry)
            case "book":
                return self.__format_book(entry)
            case "inproceedings":
                return self.__format_inproceedings(entry)
            case "online":
                return self.__format_online(entry)
            case _:
                print("WARNING: Unknown type journal")
                return ""

    def __format_article(self, entry: model.Entry):
        """Formats the article citation type"""
        final_string = ""
        if "journaltitle" in entry:
            final_string = f"{entry['journaltitle']}"
        if "volume" in entry:
            final_string += f" Vol. {entry['volume']}"
        if "number" in entry:
            final_string += f"({entry['number']})"
        if "pages" in entry:
            final_string += f", pp. {entry['pages']}"

        return final_string

    def __format_book(self, entry: model.Entry):
        """Formats the book citation type"""
        return f"{entry['publisher']}"

    def __format_inproceedings(self, entry: model.Entry):
        if "publisher" in entry:
            string = f"{entry['publisher']} "
        elif "booktitle" in entry:
            string = f"{entry['booktitle']} "
        elif "eventtitle" in entry:
            string = f"{entry['eventtitle']} "
        else:
            print("WARNING: No publisher or booktitle in the bib entry")
            return " "

        return f"{string}"

    def __format_online(self, entry: model.Entry):
        if "eprinttype" not in entry:
            print("WARNING: No eprinttype in the bib entry")
            return ""

        match entry["eprinttype"]:
            case "arxiv":
                return f"arXiv preprint arXiv:{entry['eprint']}"
            case "doi":
                return self._format_doi(entry)
            case _:
                print("WARNING: Unknown eprinttype")
                return ""



from obsidown import utils
from obsidown.operations.base import MdFile, MdOperations


class RemoveAfterString(MdOperations):
    def __init__(self, string: str):
        self.to_remove = string

    def __call__(self, file: MdFile) -> MdFile:
        """Converts the math equations from the notes into the correct format for the markdown files."""
        contents = utils.remove_after_string(file.contents, self.to_remove)
        return MdFile(
            filename=file.filename,
            metadata=file.metadata,
            contents=contents,
            references=file.references,
        )



import re
from datetime import datetime, timedelta, timezone

def interpolate_weight(dt: datetime) -> float:
    """Interpolates a weight for a given datetime between 2022 and 2030, considering full time granularity."""
    # Define start and end times

    if dt.tzinfo is not None:
        dt = dt.astimezone(timezone.utc).replace(tzinfo=None)

    start_dt = datetime(2017, 1, 1, 0, 0, 0)
    end_dt = datetime(2030, 12, 31, 23, 59, 59)

    # Define corresponding weights
    start_weight, end_weight = 1, 3_000_000

    # Ensure dt is within the valid range
    if dt < start_dt:
        return float(start_weight)
    if dt > end_dt:
        return float(end_weight)

    # Compute interpolation factor based on total seconds elapsed
    total_seconds = (end_dt - start_dt).total_seconds()
    elapsed_seconds = (dt - start_dt).total_seconds()
    factor = elapsed_seconds / total_seconds

    # Linear interpolation
    weight = start_weight + factor * (end_weight - start_weight)
    
    # Now we have to invert the process so that the smaller weight is for the most recent commit
    weight = end_weight - weight + start_weight
    return int(weight)


def parse_datetime(date_str: str):
    """
    Parse the time from the git log.
    date_str = "Mon Apr 8 01:33:22 2024 +0200"
    """
    dt = datetime.strptime(date_str[:-6], "%a %b %d %H:%M:%S %Y")
    offset_sign = 1 if date_str[-5] == '+' else -1
    offset_hours = int(date_str[-4:-2])
    offset_minutes = int(date_str[-2:])
    offset_delta = timedelta(hours=offset_hours, minutes=offset_minutes)
    dt = dt.replace(tzinfo=timezone(offset_sign * offset_delta))

    return dt


def convert_maths(page: str):
    """Convert single dollar sign math expressions to double dollar sign expressions.
    Add newline characters before and after double dollar sign expressions if they are not already present.

    """
    # Add newline characters before and after double dollar sign expressions if they are not already present
    page = re.sub(r"\$\$\n([^\$]+?)\n\$\$", r"\n$$\n\1\n$$\n", page, flags=re.DOTALL)

    # Replace single dollar sign math expressions with double dollar sign expressions
    page = re.sub(r"(?<!\$)\$([^\$]+?)\$(?!\$)", r"$$\1$$", page)

    return page


def convert_katex(page: str):
    """Converts elements found problematic into well formed katex elements

    Example
    -------
    >>> convert_katex("$$x = y \\\\$$ and $a = b$")
    "$$x = y \\\\\\$$ and $$a = b$$"
    >>> convert_katex("$$x = y_{i}$$")
    "$$x = y\\_{i}$$"
    """
    # Replace \\ with \\\\
    dollar_matches = r"\$\$([^\$]+?)\\\\([^\$\\]+?)\$\$"
    while re.search(dollar_matches, page):
        page = re.sub(dollar_matches, r"$$\1\\\\\\\2$$", page)
        print(page)

    # Replace _{ with \_{ and } with }
    substring = r"\$\$([^\$\\]+?)_{([^\$]+?)}\$\$"
    while re.search(substring, page):
        page = re.sub(substring, r"$$\1\\_{\2}$$", page)

    return page


def convert_images(page: str, base: str = ""):
    """Convert image obsidian markdown tags into html tags.
    Args
    ----
    page: str
        The page content to be converted.
    base: str
        The base url for the images.
    """
    # First convert image tags with numbers to html image tags
    page = re.sub(
        r"!\[\[([^\]]+?)(\.jpeg|\.webp|\.png|.jpg)\|([0-9|\s]+)\]\]",
        r'<img src="' + base + "/" + r'\1\2" width="\3" alt="\1">',
        page,
    )
    # Convert markdown image tags to html image tags
    page = re.sub(
        r"!\[\[([^\]]+?)(\.jpeg|\.webp|\.png|.jpg)\]\]",
        r'<img src="' + base + "/" + r'\1\2" alt="\1">',
        page,
    )

    return page


def convert_external_links(page: str):
    """If there is an external link without the markdown format, convert it to the markdown format.

    Example
    -------
    >>> convert_external_links("https://google.com")
    "[https://google.com](https://google.com)"
    >>> convert_external_links("[https://google.com](https://google.com)")
    "[https://google.com](https://google.com)"
    """

    # Questo regex è un po' fragile, ma è difficile da fare, dovresti avere lookahead infinito!
    return re.sub(
        r"(?<!\[)(https?:\/\/[^\s\]\(\)]+)(?!(\)|[a-z]|\.|[0-9]|[A-Z]|\/|_|,|-|=|\?|&|~|#|%|:))",
        r"[\1](\1)",
        page,
    )


def convert_links(page: str, base: str = ""):  #
    """Convert the links to the markdown format.
    # Warning: this assumes images to be links to!

    Example
    -------
    >>> convert_links("hello [[world]]", "https://google.com")
    "hello [world](https://google.com/world)"
    """
    # first convert hashtag links

    def convert_to_md(x):
        return "[{}]({})".format(x, to_kebab_case(x))

    def convert_two_to_md(x, y):
        return "[{}]({})".format(y, to_kebab_case(x))

    page = re.sub(
        r"\[\[(#[^\]]+?)\|([^\]]+)\]\]",
        lambda x: convert_two_to_md(x.group(1), x.group(2)),
        page,
    )
    page = re.sub(r"\[\[(#[^\]]+?)\]\]", lambda x: convert_to_md(x.group(1)), page)

    # then outer links
    def convert_to_md2(x):
        return "[{}]({}/{})".format(x, base, to_kebab_case(x))

    def convert_two_to_md2(x, y):
        return "[{}]({}/{})".format(y, base, to_kebab_case(x))

    page = re.sub(
        r"\[\[([^\]]+?)\|([^\]]+)\]\]",
        lambda x: convert_two_to_md2(x.group(1), x.group(2)),
        page,
    )
    page = re.sub(r"\[\[([^\]]+?)\]\]", lambda x: convert_to_md2(x.group(1)), page)
    return page


def filter_link(page: str, links: list[str]):
    """removes the links with  link in the page, making it a normal string

    Example
    -------
    >>> filter_link("hello [[world]]", ["world"])
    "hello world"

    """

    for link in links:
        page = re.sub(r"!?\[\[{}\]\]".format(link), link, page)

    return page


def extract_links(page: str):
    """Extract all the links from the page."""
    return re.findall(r"\[\[([^\]]+?)\]\]", page)


def is_image(name: str):
    """Check if a file is an image."""
    return (
        name.endswith(".jpeg")
        or name.endswith(".png")
        or name.endswith(".webp")
        or name.endswith(".jpg")
    )


def to_kebab_case(name: str):
    """Convert a string to kebab case."""
    return name.lower().replace("'", " ").replace(" ", "-")


def remove_extension(name: str):
    """Remove the extension from a file name."""
    if "." not in name:
        return name
    return ".".join(name.split(".")[:-1])


def remove_after_string(content: str, string: str):
    """Remove everything after a string."""
    return content.split(string)[0]
